import os
import netifaces

os.environ['SDL_VIDEODRIVER'] = 'fbcon'
os.environ['SDL_FBDEV'] = '/dev/fb1'

from kivy.app import App
from kivy.clock import Clock
from kivy.logger import Logger
from kivy.uix.button import Button
from kivy.uix.gridlayout import GridLayout
from kivy.uix.label import Label
from kivy.uix.progressbar import ProgressBar
from measure import Meter

class MyApp(App):
    def build(self):
        self.layout = GridLayout(cols=1)
        self.channels = [1,2,3,4,5,6,8]
        self.groups = [
            {
                "name": "Body Cord",
                "limit": 1,
                "channels": {
                    1: "A",
                    2: "B",
                    3: "C",
                }
            },
            {
                "name": "Weapon",
                "limit": 2,
                "channels": {4:None},
            },
            {
                "name": "Mask Cord",
                "limit": 1,
                "channels": {5:None},
            },
            {
                "name": "Fabric",
                "limit": 5,
                "channels": {6:None},
            },
        ]
        self.pb = {}
        self.labels = {}
        for channel in self.channels:
            self.pb[channel] = ProgressBar(value=0, max=2 )
            self.labels[channel] = Label(text='{0}'.format(channel), font_size=28)
            self.layout.add_widget(self.labels[channel])
            self.layout.add_widget(self.pb[channel])
        self.meter = Meter(self.channels)
        Clock.schedule_interval(self.measure, 0.01)
        Clock.schedule_interval(self.get_IP_address, 2)
        return self.layout
    def measure(self, dt):
        measurements = self.meter.measure()

        # Get the group of channels depending on what's connected
        group = None
        for channel in measurements:
            if measurements[channel]:
                group = next((g for g in self.groups if channel in g['channels']), None)
                break

        if group:
            # show measurements for the channel group containing the first connected channel we found.
            self.layout.clear_widgets()
            self.layout.add_widget(Label(text=group['name'], font_size=36))
            for ch in group['channels']:
                # Get channel name
                if group['channels'][ch]:
                    ch_name = group['channels'][ch] + ": "
                else:
                    ch_name = ""
                # See if something is connected
                if ch in measurements and measurements[ch]:
                    (ohms, volts) = measurements[ch]
                    # BG Color
                    if ohms > group['limit']:
                        bgcolor = [1,0,0,1]
                    else:
                        bgcolor = [0,1,0,1]
                    # Display the measured resistance
                    self.layout.add_widget(Label(text= u"{0}{1:.2f} \u2126".format(ch_name, ohms), font_size=36, color=bgcolor))
                else:
                    # BG Color
                    bgcolor = [1,1,1,1]
                    # Display "not connected"
                    ohms = 0
                    self.layout.add_widget(Label(text= u"{0}Not Connected".format(ch_name), font_size=36))
                self.layout.add_widget(ProgressBar(value=ohms, max=group['limit']*2, color=bgcolor))
        else:
            # Nothing was found connected
            self.layout.clear_widgets()
            self.layout.add_widget(Label(text="Connect something!", font_size=48))
            poweroff_button = Button(text="[color=ff0000]Shut Down[/color]", font_size=24, markup=True)
            poweroff_button.bind(on_press=self.poweroff)
            self.layout.add_widget(poweroff_button)
            # Display IP address if we have one
            if hasattr(self, 'ip_address_string') and self.ip_address_string:
                self.layout.add_widget(Label(
                    text="[color=3333ff]IP: {0}[/color]"\
                        .format(self.ip_address_string),
                    font_size=36,
                    markup=True
                    ))
    def get_IP_address(self, dt):
        try:
            gateway_interface = netifaces.gateways()[netifaces.AF_INET][0][1]
            ip_addresses = netifaces.ifaddresses(gateway_interface)
            internet_ip_addresses = [addr['addr'] for addr in ip_addresses[netifaces.AF_INET]]
            self.ip_address_string = ", ".join(internet_ip_addresses)
        except KeyError:
            self.ip_address_string = None
    def poweroff(self, dt):
        os.system('poweroff')

if __name__ == '__main__':
    MyApp().run()
